#include <ddraw.h>
#include <dsound.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <rnd.c>

#define KEY(x) (GetAsyncKeyState(x)>>15&1)
#define KEYUP(x) while (KEY(x) && !dealwithmessages())

LPDIRECTDRAW lpdd;
LPDIRECTSOUND lpds;
DDSURFACEDESC ddsd;
LPDIRECTDRAWSURFACE visible;
LPDIRECTDRAWPALETTE ddpal;

int xres=640,yres=480;
HWND winhandle=NULL;
unsigned char *shad,*back;
int kbhit=0,mx=0,my=0,mb=0;
char getch=1,timeup=0;

void gracefulexit()
{
	if (visible) IDirectDraw_Release(visible);
	if (lpdd) IDirectDraw_Release(lpdd);
	if (lpds) IDirectSound_Release(lpds);
}

void error(const char *message)
{
	gracefulexit();
	MessageBox(NULL,message,"Oh sod it",MB_OK);
	PostQuitMessage(0);
}

void dde(const HRESULT ddresult)
{
	if (ddresult!=DD_OK) error("DirectDraw error");
}

LRESULT CALLBACK windowproc(HWND hwnd,UINT msg,WPARAM wp,LPARAM lp)
{
	PAINTSTRUCT ps;
	kbhit=0; getch=1;
	switch (msg)
	{
		case WM_PAINT:
			BeginPaint(hwnd,&ps);
			EndPaint(hwnd,&ps);
			return 0;
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
		case WM_CHAR:
			kbhit=1;
			getch=(char)wp;
			break;
		case WM_MOUSEMOVE:
		case WM_LBUTTONUP: case WM_LBUTTONDOWN: case WM_LBUTTONDBLCLK:
		case WM_RBUTTONUP: case WM_RBUTTONDOWN: case WM_RBUTTONDBLCLK:
			mx=(int)LOWORD(lp);
			my=(int)HIWORD(lp);
			mb=(int)wp&3;
			break;
		default:;
	}
	return DefWindowProc(hwnd,msg,wp,lp);
}

HWND makethewindow(HINSTANCE hinstance)
{
	HWND ret;
	WNDCLASS winclass;

	winclass.style=CS_DBLCLKS|CS_OWNDC|CS_HREDRAW|CS_VREDRAW;
	winclass.lpfnWndProc=windowproc;
	winclass.cbClsExtra=0;
	winclass.cbWndExtra=0;
	winclass.hInstance=hinstance;
	winclass.hIcon=LoadIcon(NULL,IDI_APPLICATION);
	winclass.hCursor=LoadCursor(NULL,IDC_ARROW);
	winclass.hbrBackground=GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName=NULL;
	winclass.lpszClassName="mywin";

	if (!RegisterClass(&winclass)) error("Couldn't register window class");

//	ret=CreateWindow("mywin","Hello Dave",WS_OVERLAPPEDWINDOW|WS_VISIBLE,0,0,320,200,NULL,NULL,hinstance,NULL);
	ret=CreateWindow("mywin","NCGI Game Console",WS_POPUP|WS_VISIBLE,0,0,xres,yres,NULL,NULL,hinstance,NULL);
	ShowCursor(0);
	if (!ret) error("Couldn't create a window");
	return ret;
}

void directdrawsetup(HWND winhandle)
{
	PALETTEENTRY palette[256];
	int n;
	for (n=0;n<256;n++)
	{
		palette[n].peRed=palette[n].peGreen=0;
		palette[n].peBlue=(n*99)%256;
		palette[n].peFlags=PC_NOCOLLAPSE;
	}
	dde(DirectDrawCreate(NULL,&lpdd,NULL));
	dde(IDirectDraw_SetCooperativeLevel(lpdd,winhandle,DDSCL_ALLOWMODEX|DDSCL_FULLSCREEN|DDSCL_EXCLUSIVE|DDSCL_ALLOWREBOOT));
	dde(IDirectDraw_SetDisplayMode(lpdd,xres,yres,8));
	dde(IDirectDraw_CreatePalette(lpdd,DDPCAPS_8BIT|DDPCAPS_INITIALIZE|DDPCAPS_ALLOW256,palette,&ddpal,NULL));
	ddsd.dwSize=sizeof(ddsd);
	ddsd.dwFlags=DDSD_CAPS;
	ddsd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE;
	dde(IDirectDraw_CreateSurface(lpdd,&ddsd,&visible,NULL));
	dde(IDirectDrawSurface_SetPalette(visible,ddpal));
}

int dealwithmessages()
{
	MSG msg;
	char recover=0;
	if (IsIconic(winhandle))
	{
		if (visible) IDirectDraw_Release(visible);
		if (lpdd) IDirectDraw_Release(lpdd);
		recover=1;
	}
	while (PeekMessage(&msg,NULL,0,0,PM_REMOVE) || IsIconic(winhandle))
	{
		if (msg.message==WM_QUIT) gracefulexit();
		TranslateMessage(&msg); DispatchMessage(&msg);
	}
	if (recover) directdrawsetup(winhandle);
	return 0;
}

void makebanks()
{
	shad=(unsigned char *)malloc(xres*yres);
	back=(unsigned char *)malloc(xres*yres);
}

void ccl(const unsigned char c)
{
	int y,lp;
	unsigned char *screen;
	IDirectDrawSurface_Lock(visible,NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL);
	screen=(unsigned char *)ddsd.lpSurface;
	if ((lp=ddsd.lPitch)==xres) memcpy(screen,shad,xres*yres);
	else for (y=0;y<yres;y++) memcpy(screen+y*lp,shad+y*xres,xres);
	memset(shad,c,xres*yres);
	IDirectDrawSurface_Unlock(visible,NULL);
}

void cclb()
{
	int y,lp;
	unsigned char *screen;
	IDirectDrawSurface_Lock(visible,NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL);
	screen=(unsigned char *)ddsd.lpSurface;
	if ((lp=ddsd.lPitch)==xres) memcpy(screen,shad,xres*yres);
	else for (y=0;y<yres;y++) memcpy(screen+y*lp,shad+y*xres,xres);
	memcpy(shad,back,xres*yres);
	IDirectDrawSurface_Unlock(visible,NULL);
}

void cl(const unsigned char c)
{
	memset(shad,c,xres*yres);
	ccl(c);
}

void clb(const unsigned char c)
{
	memset(back,c,xres*yres);
}

void putpix(const unsigned x,const unsigned y,const unsigned char c)
{
	if (x<xres) if (y<yres) shad[x+y*xres]=c;
}

void putpixb(const unsigned x,const unsigned y,const unsigned char c)
{
	if (x<xres) if (y<yres) back[x+y*xres]=c;
}

unsigned char getpix(const unsigned x,const unsigned y)
{
	if (x<xres) if (y<yres) return shad[x+y*xres];
	return 255;
}

unsigned char getpixb(const unsigned x,const unsigned y)
{
	if (x<xres) if (y<yres) return back[x+y*xres];
	return 255;
}

void rect(int x,int y,int xs,int ys,const unsigned char c)
{
	int n;
	unsigned char *p;
	if (x<0) {xs+=x; if (xs<0) return; x=0;}
	if (y<0) {ys+=y; if (ys<0) return; y=0;}
	if (x>=xres) return;
	if (y>=yres) return;
	if (x+xs>xres) xs=xres-x;
	if (y+ys>yres) ys=yres-y;
	p=shad+y*xres+x;
	for (n=0;n<ys;n++)
	{
		memset(p,c,xs);
		p+=xres;
	}
}

void rectb(const unsigned x,const unsigned y,unsigned xs,const unsigned ys,const unsigned char c)
{
	int py,ymax=Min(y+ys,yres);
	unsigned char *p=back+y*xres+x;
	xs=Min(xs,xres-x);
	for (py=y;py<ymax;py++)
	{
		memset(p,c,xs);
		p+=xres;
	}
}

void wrapbit(int x,int y,const unsigned char *bitmap)
{
	unsigned xs=bitmap[0],ys=bitmap[1],px=(x<0)?x+=xres:0,py=(y<0)?y+=yres:0;
	unsigned char *c=bitmap+2,*p=shad+y*xres+x,*l=shad+yres*xres-xs;
	for (py=0;py<ys;py++)
	{
		for (px=0;px<xs;px++,c++,p++) if (*c!=255) *p=*c;
		p+=xres-xs;
		if (p>=l) p-=xres*(yres-1);
	}
}

void wrapbitb(int x,int y,const unsigned char *bitmap)
{
	unsigned xs=bitmap[0],ys=bitmap[1],px=(x<0)?x+=xres:0,py=(y<0)?y+=yres:0;
	unsigned char *c=bitmap+2,*p=back+y*xres+x,*l=back+yres*xres-xs;
	for (py=0;py<ys;py++)
	{
		for (px=0;px<xs;px++,c++,p++) if (*c!=255) *p=*c;
		p+=xres-xs;
		if (p>=l) p-=xres*(yres-1);
	}
}

void mskbit(const int x,const int y,const unsigned char *bitmap)
{
	int px,py; unsigned char *p=bitmap+2,c;
	for (py=0;py<bitmap[1];py++)
		if (y+py>=0 && y+py<yres)
		{
			for (px=0;px<bitmap[0];px++,p++)
				if ((c=*p)!=255)
					if (x+px>=0 && x+px<xres)
						shad[x+px+(y+py)*xres]=c;
		}
		else p+=bitmap[0];
}

void pal(const unsigned char n,const unsigned char r,const unsigned char g,const unsigned char b)
{
	PALETTEENTRY colour;
	colour.peRed=r; colour.peGreen=g; colour.peBlue=b;
	colour.peFlags=PC_NOCOLLAPSE;
	dde(IDirectDrawPalette_SetEntries(ddpal,0,n,1,&colour));
}

void line(float x1,float y1,float x2,float y2,const unsigned char c)
{
	float dx=x2-x1,dy=y2-y1,k,xr=xres-1,yr=yres-1;
	if (x1<0)
	{
		if (x2<0) return;
		y1-=x1*dy/dx; x1=0;
	}
	else if (x2<0)
	{
		y2-=x2*dy/dx; x2=0;
	}
	if (x1>xr)
	{
		if (x2>xr) return;
		y1-=(x1-xr)*dy/dx; x1=xr;
	}
	else if (x2>xr)
	{
		y2-=(x2-xr)*dy/dx; x2=xr;
	}
	if (y1<0)
	{
		if (y2<0) return;
		x1-=y1*dx/dy; y1=0;
	}
	else if (y2<0)
	{
		x2-=y2*dx/dy; y2=0;
	}
	if (y1>yr)
	{
		if (y2>yr) return;
		x1-=(y1-yr)*dx/dy; y1=yr;
	}
	else if (y2>yr)
	{
		x2-=(y2-yr)*dx/dy; y2=yr;
	}
	dx=x2-x1; dy=y2-y1;
	if (dx*dx+dy*dy<0.1) {shad[(int)x1+(int)y1*xres]=c; return;}
	if (dx+dy<0)
	{
		dx=-dx; dy=-dy;
		k=x1; x1=x2; x2=k;
		k=y1; y1=y2; y2=k;
	}
	if (dx>dy)
	{
		k=dy/dx;
		for (;x1<=x2;x1++)
		{
			shad[(int)x1+(int)y1*xres]=c;
			y1+=k;
		}
	}
	else
	{
		k=dx/dy;
		for (;y1<=y2;y1++)
		{
			shad[(int)x1+(int)y1*xres]=c;
			x1+=k;
		}
	}
}

void box(const unsigned x,const unsigned y,const unsigned xs,const unsigned ys,const unsigned char c)
{
	unsigned char *p=shad+y*xres+x;
	int n;
	memset(p,c,xs+1);
	memset(p+ys*xres,c,xs+1);
	for (n=y+1;n<y+ys;n++)
	{
		putpix(x,n,c);
		putpix(x+xs,n,c);
	}
}

void circlefill(const float x,const float y,const float r,const unsigned char c)
{
	int py;
	int ymin=ceil(y-r),ymax=floor(y+r);
	if (ymin<0) ymin=0;
	if (ymin>=yres) ymin=yres-1;
	if (ymax<0) ymax=0;
	if (ymax>=yres) ymax=yres-1;
	for (py=ymin;py<=ymax;py++)
	{
		float rx=sqrt(r*r-sq(py-y));
		int xmin=x-rx,xmax=x+rx;
		if (xmin<0) xmin=0;
		if (xmin>=xres) xmin=xres-1;
		if (xmax<0) xmax=0;
		if (xmax>=xres) xmax=xres-1;
		if (xmax>xmin) memset(shad+py*xres+xmin,c,xmax-xmin+1);
	}
}

void readbit(unsigned char *bmap,const char *filename,const long offset)
{
	int n,x,y;
	FILE *in=fopen(filename,"rb");
	fseek(in,offset,SEEK_SET);
	x=fgetc(in); y=fgetc(in);
	bmap[0]=x; bmap[1]=y;
	for (n=0;n<x*y;n++)
		bmap[n+2]=fgetc(in);
	fclose(in);
}

unsigned char *letter[36],*bletter[36];

void loadfonts()
{
	unsigned char twid[36]={3,3,3,3,2, 2,4,3,1,2, 4,2,5,4,3,3, 4,3,2,3,3, 4,5,3,3,3,
					  			   3,2,2,2,3,2,3,2,3,3};
	int n,i=0; // Small and Big Letters loading
	for (n=0;n<36;n++)
	{
		letter[n]=(unsigned char *)malloc(2+twid[n]*5);
		readbit(letter[n],".\\lets.dat",i);
		i+=2+twid[n]*5;
	}
	for (n=0;n<36;n++)
	{
		bletter[n]=(unsigned char *)malloc(66);
		readbit(bletter[n],".\\ters.dat",n*66);
	}
}

void write(int x,int y,const int w,const char *s,const unsigned char ch,const unsigned char cl)
{
	unsigned char twid[36]={3,3,3,3,2, 2,4,3,1,2, 4,2,5,4,3,3, 4,3,2,3,3, 4,5,3,3,3,
					  			   3,2,2,2,3,2,3,2,3,3};
	int n,i,xbeg=x,xend=w,saveaddr=(int)shad;
	unsigned char at[27]={5,5,0,1,1,1,0,1,255,255,255,1,1,255,1,1,0,1,255,255,255,255,0,1,1,1,0};
	if (ch==cl || ch==255 || cl==255) return;
	if (w==888) shad=back;
	for (n=0;n<27;n++)
	{
		if (at[n]==0) at[n]=cl;
		else if (at[n]==1) at[n]=ch;
	}
	for (n=0;n<36;n++)
		for (i=2;i<2+twid[n]*5;i++)
		{
			if (letter[n][i]==15) letter[n][i]=ch;
			else if (letter[n][i]==7) letter[n][i]=cl;
		}
	for (n=0;s[n];n++)
	{
		if (lc(s[n])>='a' && lc(s[n])<='z')
		{
			wrapbit(x,y,letter[lc(s[n])-'a']);
			x+=twid[lc(s[n])-'a']+1;
		}
		else if (s[n]>='0' && s[n]<='9')
		{
			if (s[0]=='#')
			{
				char cc=((s[n]-'1')%3==0)?cl:ch;
				line(x,y,x,y+4,cc); line(x+2,y,x+2,y+4,cc);
				putpix(x+1,y,cc); putpix(x+1,y+2,cc); putpix(x+1,y+4,cc);
				if (s[n]=='0') putpix(x+1,y+2,cl);
				else if (s[n]=='1') line(x+2,y,x+2,y+4,ch);
				else if (s[n]=='2') {putpix(x,y+1,cl); putpix(x+2,y+3,cl);}
				else if (s[n]=='3') {putpix(x,y+1,cl); putpix(x,y+3,cl);}
				else if (s[n]=='4') {line(x+2,y,x+2,y+4,ch); line(x,y,x,y+2,ch); putpix(x+1,y+2,ch);}
				else if (s[n]=='5') {putpix(x+2,y+1,cl); putpix(x,y+3,cl);}
				else if (s[n]=='6') putpix(x+2,y+1,cl);
				else if (s[n]=='7') {line(x+2,y,x+2,y+4,ch); line(x,y,x+2,y,ch);}
				else if (s[n]=='9') putpix(x,y+3,cl);
				x+=4;
			}
			else
			{
				wrapbit(x,y,letter[26+s[n]-'0']);
				x+=twid[26+s[n]-'0']+1;
			}
		}
		else if (s[n]==' ')
		{
			x+=2;
			if (x>xend) {x=xbeg; y+=6;}
		}
		else if (s[n]=='\n') {x=xbeg; y+=6;}
		else if (s[n]==39)
		{
			putpix(x,y,ch);
			putpix(x,y+1,ch);
			x+=2;
		}
		else if (s[n]=='.')
		{
			putpix(x,y+4,ch);
			x+=2;
		}
		else if (s[n]=='-')
		{
			putpix(x,y+2,ch);
			putpix(x+1,y+2,ch);
			x+=3;
		}
		else if (s[n]=='+')
		{
			putpix(x,y+2,ch);
			putpix(x+1,y+2,ch);
			putpix(x+2,y+2,ch);
			putpix(x+1,y+1,ch);
			putpix(x+1,y+3,ch);
			x+=4;
		}
		else if (s[n]=='@')
		{
			wrapbit(x,y,at);
			x+=6;
		}
		else if (s[n]=='!')
		{
			putpix(x,y,ch);
			putpix(x,y+1,ch);
			putpix(x,y+2,ch);
			putpix(x,y+4,ch);
			x+=2;
		}
		else if (s[n]=='?')
		{
			putpix(x,y,ch);
			putpix(x+1,y,cl);
			putpix(x+1,y+1,ch);
			putpix(x+1,y+2,cl);
			putpix(x,y+2,ch);
			putpix(x,y+4,ch);
			x+=3;
		}
		else if (s[n]=='[')
		{
			putpix(x,y,ch);
			putpix(x,y+1,ch);
			putpix(x,y+2,ch);
			putpix(x,y+3,ch);
			putpix(x,y+4,ch);
			putpix(x+1,y,ch);
			putpix(x+1,y+4,ch);
			x+=3;
		}
		else if (s[n]==']')
		{
			putpix(x+1,y,ch);
			putpix(x+1,y+1,ch);
			putpix(x+1,y+2,ch);
			putpix(x+1,y+3,ch);
			putpix(x+1,y+4,ch);
			putpix(x,y,ch);
			putpix(x,y+4,ch);
			x+=3;
		}
		else if (s[n]=='/')
		{
			putpix(x+2,y,ch);
			putpix(x+1,y+2,ch);
			putpix(x,y+4,ch);
			putpix(x+1,y+1,cl);
			putpix(x+2,y+1,cl);
			putpix(x,y+3,cl);
			putpix(x+1,y+3,cl);
			x+=4;
		}
		else if (s[n]==':')
		{
			putpix(x,y+2,ch);
			putpix(x,y+4,ch);
			x+=2;
		}
		else if (s[n]==',')
		{
			putpix(x,y+4,cl);
			putpix(x+1,y+3,ch);
			x+=3;
		}
	}
	if (w==888) shad=(unsigned char *)saveaddr;
	for (n=0;n<36;n++)
		for (i=2;i<2+twid[n]*5;i++)
		{
			if (letter[n][i]==ch) letter[n][i]=15;
			else if (letter[n][i]==cl) letter[n][i]=7;
		}
}

int estlength(const char *s)
{
	unsigned char twid[36]={3,3,3,3,2, 2,4,3,1,2, 4,2,5,4,3,3, 4,3,2,3,3, 4,5,3,3,3,
					  			   3,2,2,2,3,2,3,2,3,3};
	int n,ret=0;
	for (n=0;s[n];n++)
	{
		char a=lc(s[n]);
		if (a>='a' && a<='z') ret+=twid[a-'a']+1;
		else if (a>='0' && a<='9')
		{
			if (s[0]=='#') ret+=3;
			else ret+=twid[a-'0'+26]+1;
		}
		else if (a==' ') ret+=2;
		else if (a==39) ret+=2;
		else if (a=='.') ret+=2;
		else if (a=='-') ret+=3;
		else if (a=='+') ret+=4;
		else if (a=='@') ret+=6;
		else if (a=='!') ret+=2;
		else if (a=='?' || a=='[' || a==']') ret+=3;
		else if (a=='/') ret+=4;
		else if (a==':') ret+=2;
		else if (a==',') ret+=3;
	}
	return ret;
}

void bwrite(int x,int y,const int w,const char *s,const unsigned char c)
{
	int n,i,xbeg=x,xend=w;
	if (c==255) return;
	for (n=0;n<36;n++)
		for (i=2;i<66;i++)
			if (bletter[n][i]==15) bletter[n][i]=c;
	for (n=0;s[n];n++)
	{
		if (lc(s[n])>='a' && lc(s[n])<='z')
		{
			wrapbit(x,y,bletter[lc(s[n])-'a']);
			x+=9;
		}
		else if (lc(s[n])>='0' && lc(s[n])<='9')
		{
			wrapbit(x,y,bletter[26+lc(s[n])-'0']);
			x+=9;
		}
		else if (s[n]==' ')
		{
			x+=5;
			if (x>xend) {x=xbeg; y+=10;}
		}
		else if (s[n]=='\n') {x=xbeg; y+=10;}
		else if (s[n]==39)
		{
			putpix(x,y,c);
			putpix(x,y+1,c);
			x+=2;
		}
		else if (s[n]=='.')
		{
			putpix(x,y+7,c);
			putpix(x,y+6,c);
			putpix(x+1,y+7,c);
			putpix(x+1,y+6,c);
			x+=3;
		}
		else if (s[n]==':')
		{
			putpix(x,y+7,c);
			putpix(x,y+6,c);
			putpix(x+1,y+7,c);
			putpix(x+1,y+6,c);
			putpix(x,y+4,c);
			putpix(x,y+3,c);
			putpix(x+1,y+4,c);
			putpix(x+1,y+3,c);
			x+=3;
		}
		else if (s[n]==',')
		{
			putpix(x,y+7,c);
			putpix(x+1,y+6,c);
			putpix(x+1,y+7,c);
			putpix(x+2,y+6,c);
			x+=4;
		}
		else if (s[n]=='\\')
		{
			for (i=0;i<8;i++)
			{
				putpix(x+i/2,y+i,c);
				putpix(x+i/2+1,y+i,c);
			}
			x+=6;
		}
		else if (s[n]=='/')
		{
			for (i=0;i<8;i++)
			{
				putpix(x+i/2,y+7-i,c);
				putpix(x+i/2+1,y+7-i,c);
			}
			x+=6;
		}
		else if (s[n]=='_')
		{
			for (i=0;i<6;i++) putpix(x+i,y+7,c);
			x+=7;
		}
		else if (s[n]=='-')
		{
			rect(x,y+3,4,2,c);
			x+=5;
		}
		else if (s[n]=='+')
		{
			rect(x,y+3,6,2,c);
			rect(x+2,y+1,2,6,c);
			x+=7;
		}
		else if (s[n]=='#')
		{
			rect(x,y+2,8,1,c);
			rect(x,y+5,8,1,c);
			rect(x+2,y,1,8,c);
			rect(x+5,y,1,8,c);
			x+=9;
		}
		else if (s[n]=='>')
		{
			rect(x,y,3,1,c);
			rect(x+1,y+1,3,1,c);
			rect(x+2,y+2,3,1,c);
			rect(x+3,y+3,3,1,c);
			rect(x+3,y+4,3,1,c);
			rect(x+2,y+5,3,1,c);
			rect(x+1,y+6,3,1,c);
			rect(x,y+7,3,1,c);
			x+=7;
		}
		else if (s[n]=='<')
		{
			rect(x+3,y,3,1,c);
			rect(x+2,y+1,3,1,c);
			rect(x+1,y+2,3,1,c);
			rect(x,y+3,3,1,c);
			rect(x,y+4,3,1,c);
			rect(x+1,y+5,3,1,c);
			rect(x+2,y+6,3,1,c);
			rect(x+3,y+7,3,1,c);
			x+=7;
		}
		else if (s[n]=='%')
		{
			rect(x,y,2,2,c);
			rect(x+6,y+6,2,2,c);
			for (i=0;i<=6;i++)
				rect(x+i,y+6-i,2,2,c);
			x+=9;
		}
		else if (s[n]=='(' || s[n]=='[')
		{
			rect(x+2,y,2,1,c);
			rect(x+1,y+1,2,1,c);
			rect(x,y+2,2,1,c);
			rect(x,y+3,2,1,c);
			rect(x,y+4,2,1,c);
			rect(x,y+5,2,1,c);
			rect(x+1,y+6,2,1,c);
			rect(x+2,y+7,2,1,c);
			x+=5;
		}
		else if (s[n]==')' || s[n]==']')
		{
			rect(x,y,2,1,c);
			rect(x+1,y+1,2,1,c);
			rect(x+2,y+2,2,1,c);
			rect(x+2,y+3,2,1,c);
			rect(x+2,y+4,2,1,c);
			rect(x+2,y+5,2,1,c);
			rect(x+1,y+6,2,1,c);
			rect(x,y+7,2,1,c);
			x+=5;
		}
		else if (s[n]=='@')
		{
			rect(x+2,y,4,1,c);
			rect(x+2,y+7,4,1,c);
			rect(x,y+2,1,4,c);
			rect(x+7,y+2,1,3,c);
			putpix(x+1,y+1,c);
			putpix(x+1,y+6,c);
			putpix(x+6,y+1,c);
			rect(x+3,y+2,2,1,c);
			rect(x+3,y+5,4,1,c);
			rect(x+2,y+3,1,2,c);
			rect(x+5,y+3,1,2,c);
			x+=9;
		}
		else
		{
			putpix(x,y+7,12);
			putpix(x,y+6,14);
			putpix(x+1,y+7,14);
			putpix(x+1,y+6,12);
			x+=3;
		}
	}
	for (n=0;n<36;n++)
		for (i=2;i<66;i++)
			if (bletter[n][i]==c) bletter[n][i]=15;
}

int estblength(const char *s)
{
	int n,ret=0;
	for (n=0;s[n];n++)
	{
		if (lc(s[n])>='a' && lc(s[n])<='z' || lc(s[n])>='0' && lc(s[n])<='9')
			ret+=9;
		else if (s[n]==' ') ret+=5;
		else if (s[n]==39) ret+=2;
		else if (s[n]=='.' || s[n]==':') ret+=3;
		else if (s[n]==',') ret+=4;
		else if (s[n]=='/' || s[n]=='\\') ret+=6;
		else if (s[n]=='_') ret+=7;
		else if (s[n]=='-') ret+=5;
		else if (s[n]=='+') ret+=7;
		else if (s[n]=='#') ret+=9;
		else if (s[n]=='>' || s[n]=='<') ret+=7;
		else if (s[n]=='%') ret+=9;
		else if (s[n]=='(' || s[n]==')') ret+=5;
		else ret+=3;
	}
	return ret;
}

